import { CoreAdminContext, CoreAppContext, CoreLayoutProps, Route } from '@rakit/core';
import { useFetch } from '@rakit/fetch';
import { Error, Loading, ThemeProvider } from '@rakit/joy-ui';
import { createAuthProvider } from './config/createAuthProvider';
import { createDataProvider } from './config/createDataProvider';
import { i18nProvider } from './config/i18nProvider';
import { lazy, Suspense } from 'react';

const AppLoading = () => <Loading />

const Layout = (props: CoreLayoutProps) => {
  return (
    <Suspense fallback={<Loading />}>
      {props.children}
    </Suspense>
  )
}

const CatchAll = lazy(() => import("./pages/CatchAll"));
const SignIn = lazy(() => import("./pages/SignIn"));
const PageError = lazy(() => import("./pages/PageError"));

export default function App() {
  return (
    <ThemeProvider>
      <AppContext />
    </ThemeProvider>
  );
}

function AppContext() {
  const fetch = useFetch();

  const authProvider = createAuthProvider(fetch);
  const dataProvider = createDataProvider(fetch);

  return (
    <CoreAppContext
      authProvider={authProvider}
      // authCallbackPage={ }
      // basename={ }
      catchAll={<CatchAll />}
      dataProvider={dataProvider}
      error={Error}
      homepage={<CatchAll />}
      initialLocation="/"
      i18nProvider={i18nProvider}
      loading={AppLoading}
      layout={Layout}
      loginPage={SignIn}
      // ready={ }
      requireAuth={false}
      // store={ }
      title="进销存系统"
    >
      <Route
        name="pageErrpr"
        path="/error/:status"
        element={<PageError />}
      />
      <Route
        name="test"
        path="/sasda"
        element={<div>哈哈哈哈</div>}
      />
      <CoreAdminContext
        basepath="/admin"
        initialLocation="/admmin"
      >
        <Route
          name="test"
          path="/sasda"
          element={<div>哈哈222哈哈</div>}
        />
      </CoreAdminContext>
    </CoreAppContext>
  )
}
