import { UseAsyncOptions } from "@rakit/use-async";
import { ComponentType, ReactElement } from "react";

export type CanReturn = {
  can: boolean;
  reason?: string;
};

export interface AccessParamsCustoms {}

export type AccessParams = Record<string, any> & AccessParamsCustoms;

export interface CanParams {
  /**
   * Resource name for API data interactions
   */
  on?: string;
  /**
   * Intended action on resource
   */
  key: string;
  /**
   * Parameters associated with the resource
   * @type {
   *   resource?: [IResourceItem](https://refine.dev/docs/api-reference/core/interfaceReferences/#canparams),
   *   id?: [BaseKey](https://refine.dev/docs/api-reference/core/interfaceReferences/#basekey), [key: string]: any
   * }
   */
  params?: AccessParams;
}

export type CanFunction = (params: CanParams) => Promise<CanReturn> | CanReturn;

export type AccessControlOptions = Omit<
  UseAsyncOptions<CanReturn, CanParams, unknown>,
  'executor' | 'variables'
>;

export type AccessFallbackProps = {
  reason?: string;
  error?: unknown;
}

export type AccessFallbackComponent = ComponentType<AccessFallbackProps> | ReactElement<AccessFallbackProps>;

export interface AccessControlContextCustomValue {}

export interface AccessControlContextValue extends AccessControlContextCustomValue {
  can?: CanFunction;
  queryOptions?: AccessControlOptions;
  fallback?: AccessFallbackComponent;
}
