import {
  useAsync,
  UseAsyncResult,
  useAutomatic
} from "@rakit/use-async";
import {
  AccessControlOptions,
  CanParams,
  CanReturn
} from "./types";
import { useAccessControl } from "./useAccessControl";
import { useMemo } from "react";

export type UseCanProps = CanParams & {
  queryOptions?: AccessControlOptions;
}

export type UseCanResult =
  & Omit<
    UseAsyncResult<CanReturn, CanParams, unknown>,
    'execute' | 'abort' | 'data' | 'isExecuting'
  >
  & { isLoading: boolean }
  & CanReturn;

export function useCan(options: UseCanProps): UseCanResult {
  const { queryOptions, ...params } = options;
  const context = useAccessControl({});
  const can = context?.can

  const asyncOptions = {
    ...context?.queryOptions,
    ...queryOptions,
  }

  const {
    data,
    error,
    isExecuting: isLoading,
    execute,
  } = useAsync<CanReturn, CanParams, unknown>({
    ...asyncOptions,
    variables: params,
    executor: (params) => can?.(params) ?? ({ can: true }),
    immediate: typeof can !== "undefined",
  });

  useAutomatic({
    execute,
    params,
    asyncOptions,
  });

  return useMemo(() => {
    if (typeof can === "undefined") {
      return {
        can: true,
        isLoading: false,
        error: undefined,
        reason: undefined,
      }
    }
    return {
      can: data?.can ?? false,
      error,
      isLoading,
      reason: data?.reason,
    }
  }, [can, data, error, isLoading]);
}
