import {
  IndexRouteProps as IndexRoutePropsRaw,
  PathRouteProps as PathRoutePropsRaw,
} from "react-router-dom";
import { Authenticated } from "../auth";
import { RestoreScrollPosition } from "../scrollPosition";
import { ComponentType, ReactNode } from "react";
import { AccessParams, CanAccess } from "../accessControl";

export type RoutePropsBase = {
  accessParams?: AccessParams;
  authorised?: boolean;
  name: string;
  remembeScrollPosition?: boolean;
}

export type IndexRouteProps = IndexRoutePropsRaw & RoutePropsBase;
export type PathRouteProps = PathRoutePropsRaw & RoutePropsBase;
export type RouteProps = IndexRouteProps | PathRouteProps;

export function Route(props: RouteProps) {
  const {
    accessParams,
    authorised,
    Component,
    element,
    name,
    remembeScrollPosition,
  } = props;

  let children = getElement(element, Component);

  if (authorised) {
    children = (
      <CanAccess
        on="route"
        key={name}
        params={accessParams}
      >
        {getElement(element, Component)}
      </CanAccess>
    );
  }

  if (remembeScrollPosition) {
    children = (
      <RestoreScrollPosition storeKey={name}>
        {children}
      </RestoreScrollPosition>
    );
  }

  if (authorised) {
    children = (
      <Authenticated>
        {children}
      </Authenticated>
    );
  }

  return children;
}

function getElement(
  element: ReactNode | undefined | null,
  Component: ComponentType | undefined | null
) {
  if (element != null) {
    return element;
  }
  if (Component != null) {
    return (<Component />)
  }
  return null;
}
